/* apcheck1
  performs first round of checks on inputs
*/

void apcheck1()
{
   char errstg[100];
   int i,j,ok;
   double sum;
   
  // check for presence of securities matrix -- exit if matrix is missing
      if (securities == NULL) aperrorhalt("Securities table missing");
      
  // check for presence of portfolio matrix -- exit if matrix is missing
      if (portfolios == NULL) aperrorhalt("Portfolios table missing");
      
  // check for presence of probabilities matrix -- exit if matrix is missing
      if (probabilities == NULL) aperrorhalt("Probabilities table missing");
      
  // check for consistency of dimensions in securities, portfolios and probabilities matrices -- exit if inconsistent
     if (portfolioscolumns != nsecurities) aperrorhalt("Portfolios table -- number of columns does not equal the number of securities");
     if (probabilitiescolumns != nstates) aperrorhalt("Probabilities table -- number of columns does not equal the number of states");
     
 // check for proper format in securities table for security 1 -- 1 in upper left and zeros for all other entries
     if (securities[0] != 1) nerrors = 1;
     for (i=1;i<nstates;i++) 
       { 
          #ifdef MEMCHECK
              if ((nsecurities*i+0)>securitiessize) aperrorhalt("Memory error: apcheck1 01");
          #endif   
          if (securities[nsecurities*i+0] != 0) nerrors = nerrors+1; 
       }
     for (i=1;i<nsecurities;i++) 
       { 
          #ifdef MEMCHECK
              if ((nsecurities*0+i)>securitiessize) aperrorhalt("Memory error: apcheck1 02");
          #endif
          if (securities[nsecurities*0+i] != 0) nerrors = nerrors+1; 
       }
     if (nerrors > 0) aperrorhalt("Securities table: first row must have 1 followed by zeros and first column must have 1 followed by zeros");
     
 // check for proper format in securities table for security 2 -- 0 in first row followed by ones for all other entries
     #ifdef MEMCHECK
          if ((nsecurities*0+1)>securitiessize) aperrorhalt("Memory error: apcheck1 03");
     #endif
     if (securities[nsecurities*0+1] != 0) nerrors = nerrors+1; 
     for (i=1;i<nstates;i++) 
       { 
          #ifdef MEMCHECK
             if ((nsecurities*i+1)>securitiessize) aperrorhalt("Memory error: apcheck1 04");
          #endif
          if (securities[nsecurities*i+1] != 1) nerrors = nerrors+1; 
       }
     if (nerrors > 0) aperrorhalt("Securities table: second column must have 0 followed by ones");
    
// check that remaining entries in securities table are non-negative
     for (i=1;i<nstates;i++) 
       for (j=1;j<nsecurities;j++)
       { 
          #ifdef MEMCHECK
             if ((nsecurities*i+j)>securitiessize) aperrorhalt("Memory error: apcheck1 05");
          #endif      
          if (securities[nsecurities*i+j] < 0.0) nerrors = nerrors+1; 
       }
     if (nerrors > 0) aperrorhalt("Securities table: no negative entries allowed");
     
// check to insure that each future security has at least one positive entry
     for (j=1;j<nsecurities;j++)
     {
        ok  = 0;
        for (i=1;i<nstates;i++) 
        {  
           #ifdef MEMCHECK
              if ((nsecurities*i+j)>securitiessize) aperrorhalt("Memory error: apcheck1 06");
           #endif      
           if (securities[nsecurities*i+j] > 0.0) ok = 1;
        }   
        if (ok==0) aperrorhalt("Securities table: every security must have a positive payoff in at least one state");
     } 
     
// check probabilities table for allowed probabilities
   if (probabilities[0] != 1) aperrorhalt("Probabilities table: first entry must equal 1");
     
     sum = 0;
     for (i=1;i<nstates;i++) 
       { sum = sum + probabilities[i];
       }
     if ( (sum > 1.00001) || (sum < 0.99999) ) aperrorhalt("Probabilities table: sum of second through last entries must equal 1");
  
 // insure that future probabilities equal 1 
       sum = 0;
       for (i=1;i<nstates;i++) sum = sum + probabilities[i];
       for (i=1;i<nstates;i++)
       {
          #ifdef MEMCHECK
             if (i>probabilitiessize) aperrorhalt("Memory error: apcheck1 07");
          #endif      
          probabilities[i] = probabilities[i]/sum;     
       }
     
 // check for probabilities outside of range from 0 to 1      
     nerrors = 0;
     for (i=1;i<nstates;i++) 
       { 
          #ifdef MEMCHECK
             if (i>probabilitiessize) aperrorhalt("Memory error: apcheck1 08");
          #endif      
          if ((probabilities[i] <= 0.0) || (probabilities[i] > 1.0) ) nerrors = nerrors+1;
       }
     if (nerrors>0) aperrorhalt("Probabilities table: each probability must be between 0 and 1, inclusive");
     
} 