/* apcheckdp
  checks discounts and utilities tables and expands if necessary
*/

void apcheckdp()
{
   int i,j;
   double *olddiscounts=NULL;    
         int olddiscountsrows; int olddiscountscolumns;      
   int typ,npos,fneg,maxlength,lgth,nsegments;
         
 // check for presence of discounts table -- exit if missing
    if (discounts == NULL) aperrorhalt("Discounts table missing");
    
 // check number of rows in discounts table
    if (discountsrows != ninvestors) aperrorhalt("Number of rows in discounts table does not equal number of investors");

 // if discounts matrix has only one column, expand it
     if (discountscolumns == 1)
     {
        olddiscounts = discounts;
        discounts = new double[ninvestors*nstates];  
        discountsrows = ninvestors;
        discountscolumns = nstates;
        discountssize = ninvestors*nstates;
        for (i=0;i<ninvestors;i++)
        {
          
          #ifdef MEMCHECK
             if ((i*nstates+0)>discountssize) aperrorhalt("Memory error: apcheckdp 01");
          #endif      
          discounts[i*nstates+0] = 1;
          for (j=1;j<nstates;j++)
          {
            #ifdef MEMCHECK
               if ((i*nstates+j)>discountssize) aperrorhalt("Memory error: apcheckdp 02");
            #endif      
            discounts[i*nstates+j] = olddiscounts[i];
          }
        }
     }
 
     
   // check for 1s in first column of discounts table
      nerrors = 0;
      for (i=0;i<ninvestors;i++) 
      {
         #ifdef MEMCHECK
            if ((i*nstates+0)>discountssize) aperrorhalt("Memory error: apcheckdp 03");
         #endif      
         if (discounts[i*nstates+0] != 1) nerrors = nerrors +1;
      }
       if (nerrors>0) aperrorhalt("Discounts table: all entries in the first column must equal 1");
      
   // check for positive values in remainder of discounts table
      nerrors = 0;
      for (i=0;i<ninvestors;i++) 
      {
          for (j=1;j<nstates;j++)
          {
             #ifdef MEMCHECK
               if ((i*nstates+j)>discountssize) aperrorhalt("Memory error: apcheckdp 04");
             #endif      
             if (discounts[i*nstates+j] <= 0) nerrors = nerrors +1;
          }
       }   
       if (nerrors>0) aperrorhalt("Discounts table: all entries must be positive");
   
  // check for presence of utilities table -- exit if missing
    if (utilities == NULL)aperrorhalt("Utilities table missing");

  // check number of rows in utilities table
    if (utilitiesrows != ninvestors) aperrorhalt("Number of rows in utilities table does not equal number of investors");
   
  // check formats of utilities table rows and prepare mutils table 
    maxlength = 0;
    for (i=0;i<ninvestors;i++)
    { 
      // find first negative, 0 or blank
         fneg = -1;
         for (j=0;j<utilitiescolumns;j++)
           if ((utilities[utilitiescolumns*i+j]<=0)&&(fneg<0))
           {
             fneg = j;
           }
      // number of positive entries before first negative, 0 or blank (including type)     
         if (fneg == -1)npos = utilitiescolumns;
         if (fneg != -1) npos = fneg;
    
     // get type
         typ = utilities[utilitiescolumns*i+0];
         
         if ((npos<2)&&(typ<4)) aperrorhalt("Utilities table: Each row must have at least two positive entries");
        
   
      // find max length for mutils table and exit if any row has a type other than 1, 2, 3 or 4   
         switch (typ) 
         {
          case 1:
            if (npos != 2) aperrorhalt("Utilities table: rows of type 1 must have one parameters");
            lgth = 2; 
            if (lgth>maxlength) maxlength = lgth;
            break;
            
          case 2:
            if (npos != 2) aperrorhalt("Utilities table: rows of type 2 must have one parameter");
            lgth = 2;
            if (lgth>maxlength) maxlength = lgth;
            break;
           
          case 3:
           nsegments = (npos+1)/2;
           lgth = 2 + 4*nsegments; 
           if (lgth>maxlength) maxlength = lgth;
           break;

          case 4:
            lgth = 3;
            if (lgth>maxlength) maxlength = lgth;
            break;
           
          default:
            aperrorhalt("Utilities table: type must be 1, 2, 3 or 4");
        }    
    } 

  // create mutils table and fill with zeros
       mutilscolumns = maxlength;
       mutilsrows = ninvestors;
       mutils = new double[ninvestors*mutilscolumns];  
       mutilssize = ninvestors*mutilscolumns;
       for (i=0;i<ninvestors;i++)
         for (j=0;j<mutilscolumns;j++)
         {  
           #ifdef MEMCHECK
              if ((mutilscolumns*i+j)>mutilssize) aperrorhalt("Memory error: apcheckdp 05");
           #endif      
           mutils[mutilscolumns*i+j] = 0;
         }  
    } 