/* apdorounds.cpp
   do rounds of trading
*/

/*
   algorithm:
     1. all discounted expected marginal utilities are positive and finite
     2. therefore all reservation prices are positive and finite
     3. two reservation prices are used, a buy and a sell reservation price
     4. initially both are set to the reservation price
     5. if the credit check would be violated by a purchase, the buy reservation price is set at -infinity
     6. if the credit check would be violated by a sale, the sell reservation price is set at infinity
     7. trades can take place if the maximum buy price exceeds the minimum sell price
     8. the goal is to trade until every feasible range of prices is below or equal to the specified maximum percentage price range
     9. a round of trades proceeds from the first future security through the last
     10. rounds are conducted until the total amount traded in a round equals zero
     11. in each round, the amount traded is set to zero if the feasible price range is below or equal to the maximum percentage
     12. if trade is feasible for a security, the price is set using the price discovery procedure determined by controls.
     13. if trade is feasible for a security, only securities with reservation prices equal to or farther from the price 
          than 1/2 the maximum allowed percentage amount are traded. This insures that a trade will be made.
     14. At end of trading, if any security price is still infinity, it is set to the average of all reservation prices
  */
 
double apdorounds(int sw)
{
  int finished,sec,inv;
  double totaltrades,sum,alltrades;
  double totaltime, cumtime;
  clock_t oldclock;
  
  // print to show trading
     if (sw>0) printf("trading");
     
  // set done to false
     finished=0;
     
  // set alltrades to zero 
     alltrades = 0;
     
  // set old clock
     oldclock = clock();
     
  // do rounds until finished
    while (finished==0)
    {
      // do a round of trading by making a market (if possible) for every future security
         totaltrades = 0;
         for (tradesecurity=1;tradesecurity<nsecurities;tradesecurity++)
         {
            apmakemarket2();
            totaltrades =  totaltrades + tradeamount;
          }
         
      // add trades in this round to total trades    
          alltrades = alltrades + totaltrades;
  
     // print if more than a second has passed since the last print
         cumtime = (clock()-oldclock)/CLOCKS_PER_SEC;
         if (cumtime > .1)
         {
           if (sw>0) printf(".");
           oldclock = clock();
         }
         
         
      // if the total amount traded is zero, no more rounds needed
         if (totaltrades == 0) finished=1;
    } 
    
    if (sw>0) printf("\n");
    
  // if any security prices are still infinity, set to average of all reservation prices
      for (sec=1;sec<nsecurities;sec++)
         if (securityprices[sec] == INFINITY)
         {
           sum = 0.0;
           for(inv=0;inv<ninvestors;inv++)
             sum = sum + respsecurities[nsecurities*inv+sec];
           #ifdef MEMCHECK
              if (sec>securitypricessize) aperrorhalt("Memory error: apdorounds 01");
           #endif   
           securityprices[sec] = sum/ninvestors;  
         }
      
  
  // return all trades
      return alltrades;
 }