/* apgetchars.cpp
   gets security, portfolio and market characteristics
   note: returns are for future states (all but first) and securities are future securities (all but first)
   note: first future security must be a riskless bond
*/

void apgetchars()
{
  int nfsecurities, nfstates;
  int sec,inv,state,chr;
  int nchars;
  double mval,rfreturn,dev,ex,devy,devx;
  double meanmktpower,covmmktpower;
  int i;
  char c;
  
  // set number of columns in securitychars and portfoliochars
      nchars = 8;
      
  // number of future securities and states
      nfsecurities = nsecurities -1;
      nfstates    = nstates - 1;
      
  // ============== security characteristics ====================    
      
  // security expected returns  in column 1
     for (sec=0;sec<nfsecurities;sec++)
       {
          mval = 0;
          for (state=0;state<nfstates;state++)
            {
               mval = mval + securityreturns[nfstates*sec+state]*probabilities[state+1];  
            }
           #ifdef MEMCHECK
               if ((nchars*sec+0)>securitycharssize) aperrorhalt("Memory error: apgetchars 01");
           #endif   
           securitychars[nchars*sec+0] = mval;
       }
       
    // riskfree return   
        rfreturn = securitychars[0];

    // expected excess returns in column 2
       for (sec=0;sec<nfsecurities;sec++)
         {
             #ifdef MEMCHECK
               if ((nchars*sec+1)>securitycharssize) aperrorhalt("Memory error: apgetchars 02");
            #endif   
            securitychars[nchars*sec+1] = securitychars[nchars*sec+0]-rfreturn;
         }   

    // standard deviations of return in column 3
        for (sec=0;sec<nfsecurities;sec++)
          {
             mval = 0;
             for (state=0;state<nfstates;state++)
               {
                 dev = securityreturns[nfstates*sec+state] - securitychars[nchars*sec+0]; 
                 mval = mval + (probabilities[state+1]*(dev*dev));  
               }  
              #ifdef MEMCHECK
                 if ((nchars*sec+2)>securitycharssize) aperrorhalt("Memory error: apgetchars 03");
              #endif   
              securitychars[nchars*sec+2] = sqrt(mval);
          }   

    // Sharpe Ratios in column 4
       for (sec=0;sec<nfsecurities;sec++)
         {
            if (securitychars[nchars*sec+2] ==0)
               {
                 #ifdef MEMCHECK
                    if ((nchars*sec+3)>securitycharssize) aperrorhalt("Memory error: apgetchars 04");
                 #endif   
                 securitychars[nchars*sec+3]=0;
               } 
            else
               {
                 #ifdef MEMCHECK
                    if ((nchars*sec+3)>securitycharssize) aperrorhalt("Memory error: apgetchars 05");
                 #endif   
                 securitychars[nchars*sec+3] = securitychars[nchars*sec+1]/securitychars[nchars*sec+2];
               } 
         }   

  // ================= market characteristics ======================
  
    // market expected return, expected excess return, sd of return and Sharpe ratio
      // market expected return and expected excess return
          ex = 0;
          for (state=0;state<nfstates;state++)  ex = ex + (marketreturns[state]-rfreturn)*probabilities[state+1];
          marketchars[0] = ex + rfreturn;
          #ifdef MEMCHECK
             if (7>marketcharssize) aperrorhalt("Memory error: apgetchars 06");
          #endif   
          marketchars[1] = ex;
      // market sd
          mval=0;
          for (state=0;state<nfstates;state++)
            { 
              dev = marketreturns[state] - marketchars[0];
              mval = mval + (probabilities[state+1]*(dev*dev)); 
            }
             marketchars[2] = sqrt(mval);
      // market Sharpe Ratio 
             if (marketchars[2]==0) 
                  marketchars[3] = 0;
               else
                  marketchars[3] =  marketchars[1]/marketchars[2];
      // market beta
          marketchars[4] = 1;
      // market alpha
          marketchars[5] = 0;
      // market power beta
          marketchars[6] = 1;
      // market power alpha
          marketchars[7] = 0;
       

 // ================== security beta and alpha values ===================
 
 // beta values in column 5    
       for (sec=0;sec<nfsecurities;sec++)
          {
             mval = 0;
             for (state=0;state<nfstates;state++)
               {
                 devy = securityreturns[nfstates*sec+state] - securitychars[nchars*sec+0]; 
                 devx =  marketreturns[state] - marketchars[0];
                 mval = mval + (probabilities[state+1]*(devx*devy));  
               }  
             #ifdef MEMCHECK
                if ((nchars*sec+4)>securitycharssize) aperrorhalt("Memory error: apgetchars 07");
             #endif   
             securitychars[nchars*sec+4] = mval/(marketchars[2]*marketchars[2]);
          }   
          
  // alpha values in column 6        
       mval = marketchars[1];
       for (sec=0;sec<nfsecurities;sec++)
         {
            #ifdef MEMCHECK
               if ((nchars*sec+5)>securitycharssize) aperrorhalt("Memory error: apgetchars 07");
            #endif   
           securitychars[nchars*sec+5] = securitychars[nchars*sec+1] - securitychars[nchars*sec+4]*mval;
         }
         
 // ================== security power beta and power alpha values ===================

 // get mean value of market return to smlpower 
    meanmktpower = 0;
    for (state=0;state<nfstates;state++)
    {
       meanmktpower = meanmktpower + (probabilities[state+1]*pow(marketreturns[state],smlpower));
    }
 
 // get covariance of market return with market return to sml power
    covmmktpower = 0;
    for (state=0;state<nfstates;state++)
    {  
       devy = marketreturns[state] - marketchars[0];
       devx = pow(marketreturns[state],smlpower) - meanmktpower;
       covmmktpower = covmmktpower + (probabilities[state+1]*devx*devy);
    }
 
 // power beta values in column 7    
       for (sec=0;sec<nfsecurities;sec++)
          {
             mval = 0;
             for (state=0;state<nfstates;state++)
               {
                 devy = securityreturns[nfstates*sec+state] - securitychars[nchars*sec+0]; 
                 devx =  pow(marketreturns[state],smlpower) - meanmktpower;
                 mval = mval + (probabilities[state+1]*(devx*devy));  
               }  
             #ifdef MEMCHECK
                if ((nchars*sec+6)>securitycharssize) aperrorhalt("Memory error: apgetchars 08");
             #endif   
                securitychars[nchars*sec+6] = mval/covmmktpower;
          }   
          
  // power alpha values in column 8        
       mval = marketchars[1];
       for (sec=0;sec<nfsecurities;sec++)
         {
            #ifdef MEMCHECK
                if ((nchars*sec+7)>securitycharssize) aperrorhalt("Memory error: apgetchars 09");
            #endif   
           securitychars[nchars*sec+7] = securitychars[nchars*sec+1] - securitychars[nchars*sec+6]*mval;
         }
                  

    // ============= portfolio characteristics =========================
  
   // portfolio expected returns  in column 1
     for (inv=0;inv<ninvestors;inv++)
       {
          mval = 0;
          for (state=0;state<nfstates;state++)
            {
               mval = mval + portfolioreturns[nfstates*inv+state]*probabilities[state+1];  
            }
            #ifdef MEMCHECK
                if ((nchars*inv+0)>portfoliocharssize) aperrorhalt("Memory error: apgetchars 10");
            #endif   
            portfoliochars[nchars*inv+0] = mval;
      }
       
     
    // expected excess returns in column 2
       for (inv=0;inv<ninvestors;inv++)
         {
            #ifdef MEMCHECK
                if ((nchars*inv+1)>portfoliocharssize) aperrorhalt("Memory error: apgetchars 11");
            #endif   
            portfoliochars[nchars*inv+1] = portfoliochars[nchars*inv+0]-rfreturn;
         }   

    // standard deviations of return in column 3
        for (inv=0;inv<ninvestors;inv++)
          {
             mval = 0;
             for (state=0;state<nfstates;state++)
               {
                 dev = portfolioreturns[nfstates*inv+state] - portfoliochars[nchars*inv+0]; 
                 mval = mval + (probabilities[state+1]*(dev*dev));  
               }  
               #ifdef MEMCHECK
                   if ((nchars*inv+2)>portfoliocharssize) aperrorhalt("Memory error: apgetchars 12");
               #endif   
               portfoliochars[nchars*inv+2] = sqrt(mval);
          }   

    // Sharpe Ratios in column 4
       for (inv=0;inv<ninvestors;inv++)
         {
             #ifdef MEMCHECK
                 if ((nchars*inv+3)>portfoliocharssize) aperrorhalt("Memory error: apgetchars 13");
             #endif   
            if (portfoliochars[nchars*inv+2] ==0)
                portfoliochars[nchars*inv+3]=0;
            else
                portfoliochars[nchars*inv+3] = portfoliochars[nchars*inv+1]/portfoliochars[nchars*inv+2];
         }   
         
   // beta values in column 5    
       for (inv=0;inv<ninvestors;inv++)
          {
             mval = 0;
             
             for (state=0;state<nfstates;state++)
               {
                 devy = portfolioreturns[nfstates*inv+state] - portfoliochars[nchars*inv+0]; 
                 devx =  marketreturns[state] - marketchars[0];
                 mval = mval + (probabilities[state+1]*(devx*devy));  
               }  
             #ifdef MEMCHECK
                 if ((nchars*inv+4)>portfoliocharssize) aperrorhalt("Memory error: apgetchars 14");
             #endif   
             portfoliochars[nchars*inv+4] = mval/(marketchars[2]*marketchars[2]);
          }   
  
   // alpha values in column 6        
       mval = marketchars[1];
       for (inv=0;inv<ninvestors;inv++)
         {
             #ifdef MEMCHECK
                 if ((nchars*inv+5)>portfoliocharssize) aperrorhalt("Memory error: apgetchars 15");
             #endif   
             portfoliochars[nchars*inv+5] = portfoliochars[nchars*inv+1] - portfoliochars[nchars*inv+4]*mval;
         }

  // power beta values in column 7    
       for (inv=0;inv<ninvestors;inv++)
          {
             mval = 0;
             for (state=0;state<nfstates;state++)
               {
                 devy = portfolioreturns[nfstates*inv+state] - portfoliochars[nchars*inv+0]; 
                 devx = pow(marketreturns[state],smlpower) - meanmktpower;
                 mval = mval + (probabilities[state+1]*(devx*devy));  
               }  
               #ifdef MEMCHECK
                  if ((nchars*inv+6)>portfoliocharssize) aperrorhalt("Memory error: apgetchars 16");
               #endif   
               portfoliochars[nchars*inv+6] =  mval/covmmktpower;;
          }   
          
  // power alpha values in column 8        
       mval = marketchars[1];
       for (inv=0;inv<ninvestors;inv++)
         {
           #ifdef MEMCHECK
               if ((nchars*inv+7)>portfoliocharssize) aperrorhalt("Memory error: apgetchars 17");
           #endif   
           portfoliochars[nchars*inv+7] = portfoliochars[nchars*inv+1] - portfoliochars[nchars*inv+6]*mval;
         }
                  

 

   }
   
       