/* apgetinvrps.cpp
   for a investor inv with consumption invcons, compute reservation price or prices
   if claimstate = 0, compute all reservation prices and return 0
   if claimstate > 0, compute reservation prices for claimstate and return it
*/

double apgetinvrps(int inv, int claimstate)
{
   int typ,state,discpredrow, mutilsrow,i,j;
   double a,b,rp,cons0,xbar,r,x,rp0;
   double xb,xup,m,m0,logm,vv,harabase;
   int seg,nsegments,base,inc,endstate;
   char c;
   
   // set parameters for case in which all reservation prices are desired
      if (claimstate == 0)
      {
         endstate = nstates;
         inc = 1;
      }
      
   // set parameters for case in which only one reservation price is desired
      if (claimstate > 0)
      {
         endstate = claimstate+1;
         inc = claimstate;
      }
      
   
   // get row in discpred table for investor
      discpredrow = nstates*inv;
      
   // get row in mutils table for investor
      mutilsrow = mutilscolumns*inv;
      
   // get investor type
      typ = mutils[mutilsrow+0];
       
   // compute state reservation prices
      switch (typ)
      {
        case 1:
          // get parameters (x<xbar: m=100-99*(x/xbar); x>=xbar: m=(x-xbar)^(-r)
             xbar = mutils[mutilsrow+1];
             r = mutils[mutilsrow+2];
             x = invcons[0];
             if (x < xbar) rp0 = 100.0 - 99.0*(x/xbar); 
             if (x >= xbar) rp0 = pow((x - xbar +1),-r);
             for (state=0;state<endstate;state=state+inc)
             {
               x = invcons[state];
               if (x < xbar) rp = 100.0 - 99.0*(x/xbar); 
               if (x >= xbar) rp = pow((x - xbar +1),-r);
               #ifdef MEMCHECK
                  if (state>invrpssize) aperrorhalt("Memory error: apgetinvrps 01");
               #endif   
               invrps[state] = discpred[discpredrow+state]*(rp/rp0);                  
             }   
             break;
             
        case 2:
          // get parameters (mu = C^-b)
             b = mutils[mutilsrow+1];
             cons0 = invcons[0];
             for (state=0;state<endstate;state=state+inc)
             {
                rp = pow((invcons[state]/cons0),-b);
                #ifdef MEMCHECK
                  if (state>invrpssize) aperrorhalt("Memory error: apgetinvrps 02");
                #endif   
                invrps[state] = discpred[discpredrow+state]*rp;
             }   
             break;
             
        case 3:
          // piecewise CRRA
            nsegments = mutils[mutilsrow+1];
            for (state=0;state<endstate;state=state+inc)
            {   
                x = invcons[state];
                base = mutilsrow +2;
                for (seg=0;seg<nsegments;seg++)
                {
                  a   = mutils[base];
                  r   = mutils[base + 1];
                  xb  = mutils[base + 2];
                  xup = mutils[base + 3];
                  if (x<xup)
                  {
                     logm = a - r*log(x/xb);
                     m = exp(logm);
                     break;
                  }
                  base = base + 4;
                }
                if (state ==0) m0 = m;
                rp = m/m0;
                #ifdef MEMCHECK
                  if (state>invrpssize) aperrorhalt("Memory error: apgetinvrps 03");
                #endif   
                invrps[state] = discpred[discpredrow+state]*rp;
            }   
            break;
            
         case 4:
          // get parameters (mu = (C- harabase)^-b)
             harabase = mutils[mutilsrow+1];
             b    = mutils[mutilsrow+2];
             cons0 = invcons[0];
             for (state=0;state<endstate;state=state+inc)
             {
                rp = pow(((invcons[state]-harabase)/(cons0-harabase)),-b);
                #ifdef MEMCHECK
                  if (state>invrpssize) aperrorhalt("Memory error: apgetinvrps 04");
                #endif   
                invrps[state] = discpred[discpredrow+state]*rp;
             }   
             break;

      }
      
   // return reservation price if only one requested (0 otherwise)  
      rp = 0;
      if (claimstate > 0) rp = invrps[claimstate];   
      return rp;   
}