/* apgetstateprices.cpp
   get the the state prices for all states by opening markets for state claims but making no trades
*/

/* 
   algorithm:
     this gets the state prices for each state by determining the price that would be set if a market were opened for that state claim
     initially, each investor's buy price and sell price is set equal to the reservation price, based on the ratio of the
        marginal utility for a unit of the claim divided by the marginal utility of consumption
     constraints on minimum consumptions are used to set buy prices to -infinity if no units can be bought and to set
        sell prices to +infinity if no units can be sold
     the maximum feasible buy price is determined along with the minimum feasible sell price
     the state price is set equal to the average of (1) the mean of the buy prices within the feasible range and (2) the mean
        of the sell prices within the feasible range
     if no trade is possible the state price is set to the mean of all investors' reservation prices
*/

void apgetstateprices()
{
   int inv,state,consrow,invrow,claimstate,numb,nums;
   double v,maxpbuy,minpsell,sumb,sums,meanb,means,sum;
   char c;
   
   // set first price to 1
      stateprices[0] = 1; 
       
   // do remaining states
    for (claimstate=1;claimstate<nstates;claimstate++)
    {
        for (inv=0;inv<ninvestors;inv++)
        {
          // set rows
             consrow = nstates*inv;
             invrow = nstates*inv;
          // set buy and selll prices to reservation state price for claimstate
             #ifdef MEMCHECK
                 if ((invrow+claimstate)>respstatessize) aperrorhalt("Memory error: apgetstateprices 01");
             #endif   
             v = respstates[invrow+claimstate];
             #ifdef MEMCHECK
                 if (inv>buypsize) aperrorhalt("Memory error: apgetstateprices 02");
             #endif   
             buyp[inv] = v;
             #ifdef MEMCHECK
                 if (inv>sellpsize) aperrorhalt("Memory error: apgetstateprices 03");
             #endif   
             sellp[inv] = v;
         // if current consumption is at minimum, no purchase is feasible, set buyp to -infinity (no purchase at any price)
             if (consumptions[consrow+0]<=minconsumptions[consrow+0]) buyp[inv] = -INFINITY;
          // if consumption in claimstate is at minimum, no sale is feasible
             if (consumptions[consrow+claimstate]<=minconsumptions[consrow+claimstate])
                  sellp[inv] = INFINITY;
        }        
   
        // find maximum buy price and minimum sell price
            maxpbuy  = buyp[0];
            minpsell = sellp[0];
            for (inv=1;inv<ninvestors;inv++)
            { 
               if (buyp[inv]>maxpbuy)   maxpbuy  = buyp[inv];
               if (sellp[inv]<minpsell) minpsell = sellp[inv];
            }
      
       // if trade is not possible, set price to average of all reservation prices
          if (maxpbuy < minpsell)
          {
            sum  = 0;
            for (inv=0;inv<ninvestors;inv++) 
            {
              invrow = nstates*inv;
              sum = sum + respstates[invrow+claimstate];
            }  
            #ifdef MEMCHECK
                if (claimstate>respstatessize) aperrorhalt("Memory error: apgetstateprices 04");
            #endif   
            respstates[claimstate] = sum/ninvestors;
          }
          
      // if trade is possible, set price to average of mean buy and mean sell within feasible range
         if (maxpbuy >= minpsell)
         {
           // compute mean of buy prices in range and mean of sell prices in range
              sumb = 0; numb = 0; sums = 0; nums = 0;
              for (inv=0;inv<ninvestors;inv++)
              {  if (buyp[inv]<=maxpbuy)  { sumb = sumb+buyp[inv]; numb = numb+1;}
                 if (sellp[inv]>=minpsell) { sums = sums+sellp[inv]; nums=nums+1; }
              }
              meanb = sumb/numb;
              means = sums/nums;
          // set sate price to mean of mean of buys and mean of sells within range
             #ifdef MEMCHECK
                if (claimstate>statepricessize) aperrorhalt("Memory error: apgetstateprices 05");
             #endif   
             stateprices[claimstate]  = (meanb+means)/2;
         }    
     }    
 }