/* apmakeutils
  makes mutils table with marginal utility formulas
  note: mutils table has been created and utilities table checked in apcheckdp
*/

void apmakeutils()
{
   int i,j,k,npos,fneg,typ,lgth;
   int nsegments,ucol,mcol;
   double a,r,x,lastx,xbar,b;
   double a1,b1,x1,a2,b2,temp,m1,lasta,lastr; 
   double xup,xb,xL,aL;
   
   for (i=0;i<ninvestors;i++)
    { 
        // find number of entries in utilities table
        // find first negative, 0 or blank
           fneg = -1;
           for (j=0;j<utilitiescolumns;j++)
             if ((utilities[utilitiescolumns*i+j]<=0)&&(fneg<0))
             {
               fneg = j;
             }
        // number of positive entries before first negative, 0 or blank (including type)     
           if (fneg == -1)npos = utilitiescolumns;
           if (fneg != -1) npos = fneg;

     // get type
         typ = utilities[utilitiescolumns*i+0];
          
     // make row in mutils table
         switch (typ) 
         {
          case 1:   // quadratic: 1 xbar r
            /*  segment 1 (quadratic utility):
                    xbar is "near-satiation consumption"  
                    xminconsumption <= x < xbar:  m = 100 - 99*(x/xbar)
                segment 2 (CRRA) 
                    x >= xbar: m = (x-xbar+1)^(-r))
            */
            // get near-satiation consumption
                xbar = utilities[utilitiescolumns*i+1];
            // compute risk aversion so that slope of m,x curve is the same on both sides of xbar
                r =  99/xbar;
            // put in table
               #ifdef MEMCHECK
                  if ((mutilscolumns*i+2)>mutilssize) aperrorhalt("Memory error: apmakeutils 01");
               #endif      
               mutils[mutilscolumns*i+0] = 1;
               mutils[mutilscolumns*i+1] = xbar;
               mutils[mutilscolumns*i+2] = r;
              break;
            
          case 2:   // CRRA:      2  r
           // m = x^(-r) 
             r = utilities[utilitiescolumns*i+1];
             #ifdef MEMCHECK
                if ((mutilscolumns*i+1)>mutilssize) aperrorhalt("Memory error: apmakeutils 02");
             #endif      
             mutils[mutilscolumns*i+0] = 2;
             mutils[mutilscolumns*i+1] = r;
           break;
           
          case 3:   // piecewise CRRA:  r1 x1 r2 x2 ... xn rn 
           /*  0  <= x < xup:   log(m) = a - r*log(x/xb)
                m = 1 at x = x1 */
           nsegments =  (npos+1)/2;
           #ifdef MEMCHECK
              if ((mutilscolumns*i+1)>mutilssize) aperrorhalt("Memory error: apmakeutils 03");
           #endif      
           mutils[mutilscolumns*i+0] = 3;
           mutils[mutilscolumns*i+1] = nsegments;
           ucol = 1;
           mcol = 2;
           lastx = 0;
         
         // first segment
             r = utilities[utilitiescolumns*i+ucol];
               if (r<=0) aperrorhalt("Utilities table: for type 3, all risk aversion coefficients must be positive");
             xup = utilities[utilitiescolumns*i+ucol+1];
               if (xup<=0) aperrorhalt("Utilities table: for type 3, consumption values must be positive");             
             xb = xup;
             a = 0;
             #ifdef MEMCHECK
                if ((mutilscolumns*i+mcol+3)>mutilssize) aperrorhalt("Memory error: apmakeutils 04");
             #endif      
             mutils[mutilscolumns*i+mcol]   = a;
             mutils[mutilscolumns*i+mcol+1] = r;
             mutils[mutilscolumns*i+mcol+2] = xb;
             mutils[mutilscolumns*i+mcol+3] = xup;             
             ucol = ucol+2;
             mcol = mcol+4;
             aL = 0;
             xL = xup;
           
           // segments up to last  
           for (k=1;k<nsegments-1;k++)
           { 
             r = utilities[utilitiescolumns*i+ucol];
               if (r<=0) aperrorhalt("Utilities table: for type 3, all risk aversion coefficients must be positive");             
             xup = utilities[utilitiescolumns*i+ucol+1];
               if (xup<=lastx) aperrorhalt("Utilities table: for type 3, consumption levels for segments must increase in value");
             a = aL;
             xb = xL;
             #ifdef MEMCHECK
                if ((mutilscolumns*i+mcol+3)>mutilssize) aperrorhalt("Memory error: apmakeutils 05");
             #endif      
             mutils[mutilscolumns*i+mcol]   = a;
             mutils[mutilscolumns*i+mcol+1] = r;
             mutils[mutilscolumns*i+mcol+2] = xb;
             mutils[mutilscolumns*i+mcol+3] = xup;             
             ucol = ucol+2;
             mcol = mcol+4;
             aL = a - r*log(xup/xb);
             xL = xup;
           }  
           
           // last segment
             r = utilities[utilitiescolumns*i+ucol];
               if (r<=0) aperrorhalt("Utilities table: for type 3, all risk aversion coefficients must be positive");             
             xup = INFINITY;
             xb= xL;
             a = aL;           
             #ifdef MEMCHECK
                if ((mutilscolumns*i+mcol+3)>mutilssize) aperrorhalt("Memory error: apmakeutils 06");
             #endif      
             mutils[mutilscolumns*i+mcol]   = a;
             mutils[mutilscolumns*i+mcol+1] = r;
             mutils[mutilscolumns*i+mcol+2] = xb;
             mutils[mutilscolumns*i+mcol+3] = INFINITY;
           break;
           
         case 4:   // HARA:      4 b  r
           // m = (x-b)^(-r) 
             b = utilities[utilitiescolumns*i+1];
             r = utilities[utilitiescolumns*i+2];
             #ifdef MEMCHECK
                if ((mutilscolumns*i+2)>mutilssize) aperrorhalt("Memory error: apmakeutils 07");
             #endif      
             mutils[mutilscolumns*i+0] = 4;
             mutils[mutilscolumns*i+1] = b;             
             mutils[mutilscolumns*i+2] = r;
           break;
           
          default:
            aperrorhalt("Utilities table: in each row the type must be 1, 2, 3 or 4");
        }    
    } 
       
 } 