/* apsdorounds.cpp
   do rounds of trading for state claims
*/

/*
   algorithm:
     1. all discounted expected marginal utilities are positive and finite
     2. therefore all reservation prices are positive and finite
     3. two reservation prices are used, a buy and a sell reservation price
     4. initially both are set to the reservation price
     5. if the credit check would be violated by a purchase, the buy reservation price is set at -infinity
     6. if the credit check would be violated by a sale, the sell reservation price is set at infinity
     7. trades can take place if the maximum buy price exceeds the minimum sell price
     8. the goal is to trade until every feasible range of prices is below or equal to the specified maximum percentage price range
     9. a round of trades proceeds from the first future security through the last
     10. rounds are conducted until the total amount traded in a round equals zero
     11. in each round, the amount traded is set to zero if the feasible price range is below or equal to the maximum percentage
     12. if trade is feasible for a security, the price is set to the average of the maximum buy and minimum sell
     12. if trade is feasible for a security, only securities with reservation prices equal to or farther from the price 
          than 1/2 the maximum allowed percentage amount are traded. This insures that a trade will be made.
 */
 
double apsdorounds(int sw)
{
  int finished,state,inv;
  double totaltrades,alltrades,sum;
  
     if (sw>0)printf("trading claims\n");
  
  // set done to false
     finished=0;
  
  // set alltrades to zero
     alltrades = 0;
     
   // do rounds until finished
    while (finished==0)
    {
      // do a round of trading by making a market (if possible) for every future state claim (security)
         totaltrades = 0;
         for (tradesecurity=1;tradesecurity<nstates;tradesecurity++)
         {
            apsmakemarket2();
            totaltrades =  totaltrades + tradeamount;
          }
     
      // add trades to alltrades
         alltrades = alltrades + totaltrades;
         
      // if the total amount traded is zero, no more rounds needed
         if (totaltrades == 0) finished=1;
         
     // print to show a round completed
        if (sw>0) printf(".");
    } 
    
    if (sw>0) printf("\n");
    
   // if any state price is still infinity, set to average of all reservation prices
      for (state=1;state<nstates;state++)
         if (stateprices[state] == INFINITY)
         {
      //    printf("recomputing state price %d\n",state);
           sum = 0.0;
           for(inv=0;inv<ninvestors;inv++)
             sum = sum + respstates[nstates*inv+state];
           #ifdef MEMCHECK
               if (state>statepricessize) aperrorhalt("Memory error: apsdorounds 01");
           #endif   
           stateprices[state] = sum/ninvestors;  
         } 
 
   // return all trades made 
    return alltrades;
 }