/* apsim
*/

#include <stdio.h>
#include <io.h>
#include <cstring>
#include <cctype>
#include <stdlib>
#include <string.h>
#include <iostream>
#include <ctype.h>
#include <cmath>
#include <time.h>

// global variables

  // file id for error file
      FILE *fiderr;
      
  // generic bordered matrix information for reading case file
      char *bmname, *bmcolnames, *bmrownames;
      int bmnrows,bmncols,bmnsize;
      double *bmmat;
      
  // character strings
      char *statenames;          // names of states, separated and bordered by |
      char *sortedstatenames;    // names of states in sorted order, separated and bordered by |
      char *securitynames;       // names of securities, separated and bordered by !
      char *investornames;       // names of investors, separated and bordered by !
      char *fsecuritynames;      // names of future securities (all but first)
      char *fstatenames;         // names of future states (all but first)
      char *charnames;           // names of characteristics
      char *kernelnames;         // names of rows in sorted kernel and consumption table
      char *kernelnames2;        // names of rows in sorted kernel and  market return table      
      char *utilitiesnames;      // names of columns in utilities table
      char *controlsnames;       // names of parameters in controls table
      char *powerrownames;       // names of rows in power SML table
      char *powercolnames;       // names of columns in power SML table
      

  // controls
      int COMPMKT;               // 1 if complete market, 0 if not
      double MINCONSUMPTION;     // minimum allowed consumption
      int RANDSEED;              // random number seed
      double MAXPRICESPREAD;     // maximum proportional price spread
      int NUMPRICES;             // number of prices to try in price discovery
   
  // numbers 
      int nstates;               // number of states
      int nsecurities;           // number of securities
      int ninvestors;            // number of investors
      int tradesecurity;         // security to be traded
      double tradeprice;         // price for trade of tradesecurity
      double tradespread;        // spread of reservation prices for tradesecurity 
      double tradeamount;        // amount traded for a security
      double totaldemand;        // total demand for a security
      double totalsupply;        // total supply for a security
      int nerrors;               // number of errors
      double valuetraded;        // value in current consumption of a trade in a security
      double smlpower;           // power for SML beta (slope in regression of log(PPC) on log(Market Consumption)) 
      double r2power;            // r2 for regression of log(PPC) on log(Market Consumption)
       
  // vectors and dimensions
      double *tradev=NULL;            // the amount each investor will trade  (ninvestors)
         int tradevsize;
      double *resp=NULL;              // reservation prices for one security  (ninvestors)
         int respsize;
      double *buyp=NULL;              // buy prices for one security          (ninvestors) 
         int buypsize;
      double *sellp=NULL;             // sell prices for one security         (ninvestors)
         int sellpsize;
      double *invrps=NULL;            // state prices for one investor        (nstates) 
         int invrpssize;
      double *invcons=NULL;           // consumptions for one investor        (nstates) 
         int invconssize;
      double *conschg=NULL;           // consumption change from trade        (nstates) 
         int conschgsize;
      double *statevals=NULL;         // values for states (for sorting)      (nstates)
         int statevalssize;
      int    *statepos=NULL;          // positions for states (for sorting)   (nstates) 
         int statepossize;
      int    *namepos=NULL;           // positions for statename delimiters   (nstates)
         int namepossize;
      double *securityprices=NULL;    // security prices                      (nsecurities) 
         int securitypricessize;
      double *marketportfolio=NULL;   // market portfolio holdings            (nsecurities)
         int marketportfoliosize;
      double *marketconsumption=NULL; // market consumption                   (nstates)
         int marketconsumptionsize;
      double *marketsalaries=NULL;    // market salaries                      (nstates)
         int marketsalariessize;
      double *marketcollateral=NULL;  // market collateral                    (nstates)
         int marketcollateralsize;
      double *marketclaims=NULL;      // market claims                        (nstates)   
         int marketclaimssize;
      double *marketvals=NULL;        // market values                        (nsecurities)   
         int marketvalssize;
      double *marketreturns=NULL;     // market returns                       (nstates)
         int marketreturnssize;
      double *stateprices=NULL;       // means of reservation state prices    (nstates)
         int statepricessize;
      double *marketppcs=NULL;        // market ppcs                          (nstates)
         int marketppcssize;
      double *marketchars=NULL;       // market characteristics               (6) 
         int marketcharssize;
      
  // input matrices and dimensions
      double *securities=NULL;        // security payoffs                     (nstates*nsecurities)
         int securitiesrows; int securitiescolumns;int securitiessize;
      double *portfolios=NULL;        // portfolio holdings                   (ninvestors*nsecurities) 
         int portfoliosrows; int portfolioscolumns;int portfoliossize;
      double *preferences=NULL;       // investor preferences                 (ninvestors*2)
         int preferencesrows; int preferencescolumns;int preferencessize;
      double *predictions=NULL;       // investor predictions                 {ninvestors*nstates)
         int predictionsrows; int predictionscolumns;int predictionssize;      
      double *discounts=NULL;         // investor discounts                   (ninvestors*nstates)
         int discountsrows; int discountscolumns;int discountssize;      
      double *utilities=NULL;         // investor utilities                   (ninvestors*nuparameters)  
         int utilitiesrows; int utilitiescolumns;int utilitiessize;      
      double *salaries=NULL;          // investor salaries                    (ninvestors*nstates)  
         int salariesrows; int salariescolumns;int salariessize;      
      double *collateral=NULL;        // investor collateral                  (ninvestors*nstates)  
         int collateralrows; int collateralcolumns;int collateralsize;      
      double *probabilities=NULL;     // probabilities of states              (1*nstates)
         int probabilitiesrows; int probabilitiescolumns;int probabilitiessize;
      double *controls=NULL;          // control parameters                   (ncontrols*1)
         int controlsrows; int controlscolumns;int controlssize;
      double *information=NULL;       // information parameters               (ninvestors*2)
         int informationrows; int informationcolumns;int informationsize;       
         
  // computed matrices    
      double *mutils=NULL;            // formulas for marginal utilities      (ninvestors*mutilscolumns)
         int mutilscolumns;int mutilsrows; int mutilssize;
      double *consumptions=NULL;      // investor consumptions                (ninvestors*nstates) 
         int consumptionssize;
      double *portconsumptions=NULL;  // investor consumptions from portfolio (ninvestors*nstates)     
         int portconsumptionssize;
      double *claims=NULL;            // holdings of state claims             (ninvestors*nstates) 
         int claimssize;
      double *discpred=NULL;          // investor discounts*predictions       (ninvestors*nstates)
         int discpredsize;
      double *minconsumptions=NULL;   // minimum consumptions                 (ninvestors*nstates)
         int minconsumptionssize;
      double *sortedreturns=NULL;     // sorted returns                       (ninvestors*nstates); 
         int sortedreturnssize;
      double *respsecurities=NULL;    // reservation prices for securities    (ninvestors*nsecurities)
         int respsecuritiessize;
      double *portfoliovals=NULL;     // portfolio values                     (ninvestors*nsecurities) 
         int portfoliovalssize;
      double *portfolioreturns=NULL;  // portfolio returns for future states  (ninvestors*nstates-1)
         int portfolioreturnssize;
      double *securityreturns=NULL;   // security returns for future states   (ninvestors*nstates-1)
        int securityreturnssize;
      double *respstates=NULL;        // reservation prices for states        (ninvestors*nstates)       
        int respstatessize;
      double *ppcs=NULL;              // price per chance for states          (ninvestors*nstates) 
        int ppcssize;
      double *portfoliochars=NULL;    // portfolio characteristics            (ninvestors*6)
        int portfoliocharssize;
      double *securitychars=NULL;     // security characteristics             (nsecurities-1*6) 
        int securitycharssize;
      double *powerparams=NULL;       // parameters for fitted log/log kernel (2*1)
        int powerparamssize;
      double *sortedkernel=NULL;      // table for market consumption and PPCs sorted by market consumption
        int sortedkernelsize;
      
 // constants     
     #define INFINITY 1e+308   // surrogate for infinity    
     #define MEMCHECK          // checks memory use -- comment out if memory check not desired
     
 // c++ library
  #include <stdio.h>
  #include <io.h>
  #include <cstring>
  #include <cctype>
  #include <stdlib>
  #include <string.h>
  #include <iostream>
  #include <math.h>
  #include <new>

// apsim functions
   #include "aperrorhalt.cpp";
   #include "apgetbm.cpp"
   #include "apputbm.cpp"
   #include "apput2bm.cpp"   
   #include "apgetcase.cpp";
   #include "ap2detail.cpp";
   #include "apsgetcons.cpp";
   #include "apgetcons.cpp";   
   #include "apgetportcons.cpp";   
   #include "apgetinvrps.cpp";
   #include "apgetrp.cpp";
   #include "apgetrps.cpp";
   #include "apmakestd.cpp";
   #include "apgetrpk.cpp";
   #include "apsgetrpk.cpp";   
   #include "aptrade2.cpp";
   #include "apstrade2.cpp";
   #include "apmakemarket2.cpp";   
   #include "apsmakemarket2.cpp";
   #include "apdorounds.cpp";
   #include "apsdorounds.cpp";   
   #include "apgetstateprices.cpp";
   #include "apgetvals.cpp";
   #include "apsgetvals.cpp";  
   #include "apsortsv.cpp";
   #include "apsortsvret.cpp";   
   #include "apgetpower.cpp";
   #include "ap2out.cpp";
   #include "aps2out.cpp";   
   #include "apgetchars.cpp";
   #include "apcheck1.cpp";
   #include "apdopreferences.cpp";
   #include "apcheckdp.cpp";
   #include "apmakeutils.cpp";
   #include "apmakesc.cpp";   
   #include "apmakecontrols.cpp"; 
   #include "apmakepreds.cpp";
   #include "apdotrials.cpp";

void main()
{ 
   int row,col,i,j,done,lgth,ok,claimstate,inv,invrow;
   double tm,x,alltrades,consrow,sumtrades;
   char stg[110];
   int foundeof, feofid,state,finished;
   FILE *fid,*fido;
   clock_t totseconds;
   int debug;
   char c;
        
 // ------------ routines to get case, check for errors and exit if necessary, prepare all inputs and write details and error files ---
   

   // open error file, set number of errors to zero, and write error message if system or use halts program
       fiderr=fopen("c:/apsim/apsimlog.txt","w");   
       nerrors = 0;
       fprintf(fiderr,"Error\n  User or system halted program before completion\n");
       fclose(fiderr);
       
    // display header
       printf("APSIM 2.00 Creative Commons License by-nc-sa/2.5, William F. Sharpe\n");
    // get case information
       apgetcase();   
    // make first round of checks for inputs
       apcheck1();
    // if preferences table is included, prepare utilities and discounts tables
       if (preferences != NULL) apdopreferences();
    // check discounts and preferences tables
       apcheckdp();
    // check and/or make salaries and collateral tables
       apmakesc();
    // check, make and/or expand controls table
       apmakecontrols();
    // make mutils table
       apmakeutils();
    // make predictions table if needed and check format
       apmakepreds();
    // if trials required, do them and make new predictions table
       if (information!=NULL) apdotrials();
    // make remaining tables    
       apmakestd();
    // write detail file 
       ap2detail();
    // compute security and state reservation prices for all investors
       apgetrps();
    // do rounds of security trades until maximum feasible tradespread is less than threshhold
          alltrades = apdorounds(1);
          
  // section for incomplete market (controls[1] = 0)
    if (controls[1] == 0)
    {
      // get state prices by opening but not executing markets in state claims
          apgetstateprices();
      // create value and market information
          apgetvals();
      // get power for power SML
          apgetpower();
      // get portfolio, security and market characteristics
          apgetchars();
      // create output file     
          ap2out();
    } 
    
    
   // section for complete market (controls[1] = 1)
    if (controls[1] == 1)
    {
       // create matrix with consumption from portfolio securities
          apgetportcons();
       // do rounds of trades in state claims until maximum feasible tradespread is less than threshhold
          alltrades = apsdorounds(1);
       // to insure completion, re-open securities and state claims markets until no trades are completed
          finished = 0;
          while (finished==0)
          {
            sumtrades=0;
            alltrades = apdorounds(0);
            sumtrades = sumtrades+alltrades;
            alltrades = apsdorounds(0);
            sumtrades = sumtrades+alltrades;
            if (sumtrades==0) finished=1;
          }
      // do security rounds to reprice securities if needed
           alltrades = apdorounds(0);       
      // create value and market information
          apsgetvals();
      // get power for power SML
          apgetpower();
      // get portfolio, security and market characteristics
          apgetchars();
      // create output file     
          aps2out();
    } 
    
    // write error file 
        tm = clock()/CLOCKS_PER_SEC;
        tm = tm/60;
        if (nerrors==0) 
        {
          fclose (fiderr);   
          fiderr=fopen("c:/apsim/apsimlog.txt","w");            
          fprintf(fiderr,"APSIM 2.00: No errors.  Processing time: %5.3lf minutes\n", tm);
        }    
        fclose (fiderr);       
        
 } 
 
 
