/* apsmakemarket2.cpp
   make a market (if possible) for tradesecurity
*/

/* 
   algorithm:
     this makes a market in one state claim (tradesecurity)
     initially, each investor's buy price and sell price is set equal to the reservation price, based on the ratio of the
        marginal utility for a unit of the claim divided by the marginal utility of consumption
     constraints on minimum consumptions are used to set buy prices to -infinity if no units can be bought and to set
        sell prices to +infinity if no units can be sold
     the tradespread is set to the difference between the maximum buy and the minimum sell   
     if no trade is possible, no trades are made
     if trades are possible, apstrade() is called to determine the maximum amount each investor is willing and able to buy or sell
        these are signed quantities (+ for buy, - for sell) in units of the claim
     the aggregate demand and supply for the claim are then determined and the smaller (in absolute value) set to be traded
     the investors on the smaller side are allowed to trade their desired amounts, those on the larger side are allowed to 
        trade the same proportion of their desired amount so that quantity bought equals quantity sold
     after the trades are made, the consumptions matrix is updated   
*/

void apsmakemarket2()
{
   int inv,state,sec,consrow,invrow,portrow,claimstate,claimsrow,ntimes;
   double v,maxpbuy,minpsell,phigh,plow;
   double demand,supply,propdemand,propsupply,prop,amt;
   double sumb,sums,meanb,means,nethigh,netlow;
   double bestamount,pnext,netnext,pbest;
   int numb,nums,done;
   char c;
   
   // set claim state to tradesecurity
      claimstate = tradesecurity;
      
   // for each investor set buy and sell prices and adjust to infinity or -infinity if minconsumptions would be violated
        for (inv=0;inv<ninvestors;inv++)
        {
          // set rows
            invrow = nstates*inv;
            consrow = nstates*inv;
          // set invcons[0] and invcons[tradesecurity];
             invcons[0] = consumptions[invrow];
             invcons[tradesecurity] = consumptions[invrow+claimstate];
          // set buy and selll prices to reservation state price for claimstate
             v = apgetinvrps(inv,claimstate);
             buyp[inv] = v;
             sellp[inv] = v;
         // if current consumption is at minimum, no purchase is feasible, set buyp to -infinity (no purchase at any price)
             if (consumptions[consrow+0]<=minconsumptions[consrow+0]) buyp[inv] = -INFINITY;
          // if consumption in claimstate is at minimum, no sale is feasible
             if (consumptions[consrow+claimstate]<=minconsumptions[consrow+claimstate])
                  sellp[inv] = INFINITY;
        }        
 
  // find maximum buy price and minimum sell price
       maxpbuy  = buyp[0];
       minpsell = sellp[0];
       for (inv=1;inv<ninvestors;inv++)
       { 
          if (buyp[inv]>maxpbuy)   maxpbuy  = buyp[inv];
          if (sellp[inv]<minpsell) minpsell = sellp[inv];
        }
      
   // set initial high and low prices
       phigh = maxpbuy;
       plow =  minpsell;
       
   // if no trades are feasible, terminate
       if (phigh <= plow)
       {
         tradeamount = 0;
         return;
       }
   
   /*
   // find net demand for tradesecurity at high and low prices
        tradeprice = phigh;    
           nethigh = apstrade2();
        tradeprice = plow;
           netlow = apstrade2();
       
   // if netlow is not larger than nethigh, terminate
      if (netlow <= nethigh) 
       { 
          tradeamount = 0;
          return;
       }
  */
  
   // compute mean of buy prices in range and mean of sell prices in range
       sumb = 0; numb = 0; sums = 0; nums = 0;
       for (inv=0;inv<ninvestors;inv++)
       {  if ((buyp[inv]<=maxpbuy) && (buyp[inv]>=minpsell))  { sumb = sumb+buyp[inv]; numb = numb+1;}
          if ((sellp[inv]>=minpsell)&& (sellp[inv]<=maxpbuy)) { sums = sums+sellp[inv]; nums=nums+1; }
       }
       meanb = sumb/numb;
       means = sums/nums;
       
  // iterate until done
     done = 0;
     ntimes = 0;
     bestamount = -1;
     while (done == 0)
     {
       // compute next price as average of mean of buy and mean of sell prices in range if first time through
          if (ntimes == 0) pnext = (meanb + means) / 2;
       // if not first time through, compute next price as average of high and low
          if (ntimes >= 1) pnext = 0.5*(phigh + plow);
          
      // find net amount at pnext
          tradeprice = pnext;
          netnext = apstrade2();
      // save if best thus far
          if (tradeamount > bestamount)
          {
             pbest = tradeprice;
             bestamount  = tradeamount;
          }
          
      // increment ntimes
          ntimes = ntimes + 1;
      // set new high and low prices
         // if net amount is positive, make this price plow      
            if (netnext > 0)
            {
              plow = pnext;
              netlow = netnext;
            }
         // if net amount is negative, make this price phigh   
            if (netnext <= 0)
            {
              phigh = pnext;
              nethigh = netnext;
            }

      // conditions for termination
        // if net amount = 0 for tradeprice it is the optimal price
            if (netnext == 0) done = 1;
        // if new range is smaller than maxpricespread, set done to 1
            if ((phigh - plow) < MAXPRICESPREAD * 0.5 *(phigh + plow)) done = 1;
        // if max number of prices in price discovery is reached, set done to 1
            if (ntimes >= NUMPRICES) done = 1;

       // if done, set tradeprice to best and recalculate if necessary
           if (pbest != tradeprice)
           {
             tradeprice = pbest;
             netnext = aptrade2();
           }
         
   } // while (done == 0)   
  
    // find proportions traded   
         propdemand = 0;
         if (totaldemand > 0.0) propdemand = tradeamount/totaldemand;
         propsupply = 0;
         if (totalsupply > 0.0) propsupply = tradeamount/totalsupply;
         
    // make trades
         for (inv=0;inv<ninvestors;inv++)
         {
            claimsrow = nstates*inv;
            prop = propdemand;
            if (tradev[inv]<0) prop=propsupply;
            amt = prop*tradev[inv];
            claims[claimsrow+claimstate] = claims[claimsrow+claimstate] + amt;
            claims[claimsrow+0]          = claims[claimsrow+0]             - amt*tradeprice;
         }   
   
      // compute new consumption matrix
             apsgetcons();
             
      // compute new security and state reservation prices       
             apgetrps();
             
 }